.button-left {
  border-top-right-radius: 0px;
  border-bottom-right-radius: 0px;
  border-right-width: 0px;
  margin-right: 0px;
}

.button-center {
  border-radius: 0px;
  border-right-width: 0px;
  margin-left: 0px;
  margin-right: 0px;
}

.button-right {
  margin-left: 0px;
  border-top-left-radius: 0px;
  border-bottom-left-radius: 0px;
}

.borderless {
  border: 0px;
}

.txt-hint {
  color: mix (@theme_fg_color, @theme_bg_color, 0.3);
}

.headerbar-selection-multiple, .headerbar-selextion-multiple > * {
  background-color: #38a1d6;
  background-image: none;
}

.button-tight {
  padding: 0px;
}
